@@ -24,11 +24,11 @@
 void langmgr_reload_user_options(void);
 void langmgr_reload_user_styles(void);
 void langmgr_reload_user_highlights(void);
-GtkTextTag *langmrg_lookup_tag_highlight(const gchar *lang, const gchar *highlight);
+GtkTextTag *langmrg_lookup_tag_highlight(const gchar * lang, const gchar * highlight);
 GtkTextTagTable *langmgr_get_tagtable(void);
-Tbflang *langmgr_get_bflang_for_mimetype(const gchar *mimetype);
+Tbflang *langmgr_get_bflang(const gchar * mimetype, const gchar * filename);
 GList *langmgr_get_languages_mimetypes(void);
-gboolean langmgr_in_highlight_tags(GtkTextTag *tag);
+gboolean langmgr_in_highlight_tags(GtkTextTag * tag);
 #ifdef HAVE_LIBENCHANT
 GtkTextTag **langmgr_no_spellcheck_tags(void);
 GtkTextTag **langmgr_need_spellcheck_tags(void);
@@ -36,4 +36,4 @@ GtkTextTag **langmgr_need_spellcheck_tags(void);
 GList *langmgr_get_languages(void);
 void langmgr_init(void);
 void langmgr_cleanup(void);
-#endif /* _BFTEXTVIEW2_LANGMGR_H_ */
+#endif							/* _BFTEXTVIEW2_LANGMGR_H_ */
