@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * snippets_leaf_snr.c - plugin for snippets sidebar
  *
- * Copyright (C) 2006-2010 Olivier Sessink
+ * Copyright (C) 2006-2011 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,66 +22,62 @@
 
 #include "snippets.h"
 #include "snippets_gui.h"
-#include "../snr2.h"
-#include "../gtk_easy.h"
 #include "../bf_lib.h"
+#include "../dialog_utils.h"
+#include "../gtk_easy.h"
+#include "../snr3.h"
 
-
-gint snippets_snr_region_from_char(const xmlChar *region) {
+Tsnr3scope snippets_snr_region_from_char(const xmlChar *region) {
 	if (region) {
 		switch (region[0]) {  /* beginning, cursor, selection, allopenfiles */
 			case 'c':
-				return 1;
+				return snr3scope_cursor;
 			break;
 			case 's':
-				return 2;
+				return snr3scope_selection;
 			break;
 			case 'a':
-				return 3;
+				return snr3scope_alldocs;
 			break;
 			case 'b':
 			default:
-				return 0;
+				return snr3scope_doc;
 			break;
 		}
 	} 
-	return 0;
+	return snr3scope_doc;
 }
 
-gint snippets_snr_matchtype_from_char(const xmlChar *matchtype) {
-	if (matchtype) {
-		if (xmlStrEqual(matchtype, (const xmlChar *)"perl"))
-			return 3;
-		else if (xmlStrEqual(matchtype, (const xmlChar *)"posix"))
-			return 2;
-		else if (xmlStrEqual(matchtype, (const xmlChar *)"word"))
-			return 1;
-		else 
-			return 0;
-	}  
-	return 0;
+Tsnr3type snippets_snr_matchtype_from_char(const xmlChar *matchtype) {
+	/* since snr2 dropped support for posix, we try to run them as pcre. */
+	if (matchtype && (xmlStrEqual(matchtype, (const xmlChar *)"posix")
+						||xmlStrEqual(matchtype, (const xmlChar *)"perl")))
+		return snr3type_pcre;
+	return snr3type_string;
 }
 
 static void snippets_snr_run_from_strings(Tdocument *doc, const xmlChar *searchpat,const xmlChar *region, 
 					const xmlChar *matchtype,const xmlChar *casesens, const xmlChar *replacepat, const xmlChar *useescapechars) {
-	gint regionnum,matchtypenum,casesensnum,unescape;
+	gint casesensnum,unescape;
+	Tsnr3scope scope;
+	Tsnr3type type;
 	
-	regionnum = snippets_snr_region_from_char(region);
-	matchtypenum = snippets_snr_matchtype_from_char(matchtype);
+	scope = snippets_snr_region_from_char(region);
+	type = snippets_snr_matchtype_from_char(matchtype);
 	casesensnum = (casesens && casesens[0] == '1');
 	unescape = (useescapechars && useescapechars[0]=='1');
-	/* snr2_run_extern_replace
+	/* snr3_run_extern_replace
 	 * doc: a #Tdocument
 	 * search_pattern: #gchar* to search pattern
-	 * region: #gint, 0 = region_from_beginning, 1 = region_from_cursor, 2 = region_selection, 3 = region_all_open_files
+	 * scope: #gint, 0 = region_from_beginning, 1 = region_from_cursor, 2 = region_selection, 3 = region_all_open_files
 	 * matchtype: #gint, 0 = normal, 1 = posix, 2 = perl
 	 * is_case_sens: #gint
 	 * replace_pattern: #gchar* to replace pattern.
 	 * unescape: #gint
 	 * */
 	DEBUG_MSG("snippets_snr_run_from_strings, useescapechars=%s, unescape=%d\n",useescapechars,unescape);
-	snr2_run_extern_replace(doc, (gchar *)searchpat, 
-						regionnum,matchtypenum, casesensnum, (gchar *)replacepat,unescape);
+	snr3_run_extern_replace(doc, (gchar *)searchpat, 
+						scope,type, casesensnum, (gchar *)replacepat,unescape);
 }
 
 typedef struct {
@@ -110,7 +106,7 @@ static void snippets_snr_dialog(Tsnippetswin *snw, xmlNodePtr leaf, gint num_var
 	xmlChar *replacepat=NULL;
 	xmlNodePtr cur;
 	int i=0;
-	GtkWidget *table, *label;
+	GtkWidget *table, *label, *vbox;
 	gchar *tmpstr;
 	
 	title = xmlGetProp(leaf, (const xmlChar *)"title");
@@ -123,7 +119,8 @@ static void snippets_snr_dialog(Tsnippetswin *snw, xmlNodePtr leaf, gint num_var
 							NULL);
 	xmlFree(title);
 	gtk_dialog_set_default_response (GTK_DIALOG(sid->dialog),GTK_RESPONSE_ACCEPT);
-	gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(sid->dialog)->vbox),6);
+	vbox = gtk_dialog_get_content_area(GTK_DIALOG(sid->dialog));
+	gtk_box_set_spacing(GTK_BOX(vbox),6);
 	table = gtk_table_new(num_vars+1, 2, FALSE);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 12);
 	gtk_table_set_row_spacings(GTK_TABLE(table), 6);
@@ -135,7 +132,7 @@ static void snippets_snr_dialog(Tsnippetswin *snw, xmlNodePtr leaf, gint num_var
 			final_name = g_markup_escape_text((gchar *)name,-1);
 			sid->textentry[i] = gtk_entry_new();
 			gtk_entry_set_activates_default(GTK_ENTRY(sid->textentry[i]),TRUE);
-			bf_mnemonic_label_tad_with_alignment(final_name, sid->textentry[i], 0, 0.5, table, 0, 1, i+1, i+2);
+			dialog_mnemonic_label_in_table(final_name, sid->textentry[i], table, 0, 1, i+1, i+2);
 			gtk_table_attach(GTK_TABLE (table), sid->textentry[i], 1, 2, i+1, i+2, GTK_EXPAND|GTK_FILL, GTK_SHRINK, 0, 0);
 			xmlFree(name);
 			g_free(final_name);
@@ -159,7 +156,7 @@ static void snippets_snr_dialog(Tsnippetswin *snw, xmlNodePtr leaf, gint num_var
 	gtk_table_attach(GTK_TABLE (table), label, 0, 2, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
 	
 	sid->textentry[i] = NULL;
-	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(sid->dialog)->vbox),table);
+	gtk_container_add(GTK_CONTAINER(vbox),table);
 	
 	gtk_widget_show_all(sid->dialog);
 	
@@ -194,6 +191,7 @@ static void snippets_snr_dialog(Tsnippetswin *snw, xmlNodePtr leaf, gint num_var
 		casesens = xmlGetProp(leaf, (const xmlChar *)"casesens");
 		escapechars = xmlGetProp(leaf, (const xmlChar *)"escapechars");
 		snippets_snr_run_from_strings(snw->bfwin->current_document, (const xmlChar *)search_final,region,matchtype,casesens, (const xmlChar *)replace_final, escapechars);
+		g_free(replace_final);
 	}
 	gtk_widget_destroy(sid->dialog);
 	g_free(sid);
