@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * snippets_wizard.c - plugin for snippets sidebar
  *
- * Copyright (C) 2006,2009,2010 Olivier Sessink
+ * Copyright (C) 2006,2009-2011 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -116,9 +116,7 @@ static gpointer snippets_build_pageSnr(Tsnipwiz *snwiz, GtkWidget *dialog_action
 
 	const gchar *matchPattern[] = {
 		N_("Normal"),
-		N_("Entire word only"),
-		N_("POSIX"),
-		N_("PERL"),
+		N_("PERL")
 	};
 	GtkWidget *label;
 	gint i;
@@ -129,7 +127,7 @@ static gpointer snippets_build_pageSnr(Tsnipwiz *snwiz, GtkWidget *dialog_action
 	gtk_table_set_col_spacings(GTK_TABLE (p->table), 12);
 	gtk_box_pack_start(GTK_BOX(dialog_action), p->table, TRUE, TRUE, 0);
 	
-	label = gtk_label_new(_("Specify a search and a replace pattern. You may use %0, %1, ...%5 placeholders to ask for values when you activate this item. Give these placeholders an appropriate name on the right."));
+	label = gtk_label_new(_("Specify a search and a replace pattern. You may use %0, %1, ...%5 placeholders to ask for values when you activate this item. Give these placeholders an appropriate name on the right. (Please use %% if you need literal % in your string!)"));
 	/*gtk_label_set_use_markup(GTK_LABEL(label),TRUE);*/
 	gtk_label_set_line_wrap(GTK_LABEL(label),TRUE);
 	gtk_table_attach(GTK_TABLE(p->table),label, 0,4,0,1
@@ -143,16 +141,16 @@ static gpointer snippets_build_pageSnr(Tsnipwiz *snwiz, GtkWidget *dialog_action
 	dialog_mnemonic_label_in_table(_("Replace _with: "), p->replace, p->table, 0, 1, 2, 3);
 	gtk_table_attach(GTK_TABLE(p->table),p->replace, 1,4,2,3,GTK_FILL|GTK_EXPAND,GTK_FILL,0,0);
 	
-	p->scope = gtk_combo_box_new_text();
+	p->scope = gtk_combo_box_text_new();
 	for (i = 0; i < G_N_ELEMENTS(scope); i++) {
-		gtk_combo_box_append_text(GTK_COMBO_BOX(p->scope), scope[i]);
+		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(p->scope), _(scope[i]));
 	}
 	dialog_mnemonic_label_in_table(_("Sco_pe: "), p->scope, p->table, 0, 1, 3, 4);
 	gtk_table_attach(GTK_TABLE(p->table),p->scope, 1,2,3,4,GTK_FILL|GTK_EXPAND,GTK_SHRINK,0,0);
 	
-	p->matchtype = gtk_combo_box_new_text();
+	p->matchtype = gtk_combo_box_text_new();
 	for (i = 0; i < G_N_ELEMENTS(matchPattern); i++) {
-		gtk_combo_box_append_text(GTK_COMBO_BOX(p->matchtype), matchPattern[i]);
+		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(p->matchtype), _(matchPattern[i]));
 	}
 	dialog_mnemonic_label_in_table(_("Match Patter_n: "), p->matchtype, p->table, 0, 1, 4, 5);
 	gtk_table_attach(GTK_TABLE(p->table), p->matchtype, 1, 2, 4, 5, GTK_EXPAND|GTK_FILL,GTK_SHRINK, 0, 0);
@@ -257,7 +255,7 @@ static gint snippets_test_pageSnr(Tsnipwiz *snwiz, gpointer data) {
 	
 	/* build the leaf */
 	xmlSetProp(childn, (const xmlChar *)"title", (const xmlChar *)snwiz->name);
-	if (strlen(snwiz->description)) {
+	if (snwiz->description) {
 		xmlSetProp(childn, (const xmlChar *)"tooltip", (const xmlChar *)snwiz->description);
 	}
 	
@@ -281,12 +279,6 @@ static gint snippets_test_pageSnr(Tsnipwiz *snwiz, gpointer data) {
 			tmp = "normal";
 		break;
 		case 1:
-			tmp = "word";
-		break;
-		case 2:
-			tmp = "posix";
-		break;
-		case 3:
 			tmp = "perl";
 		break;
 	}
@@ -337,7 +329,7 @@ static gpointer snippets_build_pageInsert(Tsnipwiz *snwiz, GtkWidget *dialog_act
 	gtk_table_set_col_spacings(GTK_TABLE (p2->table), 12);
 	gtk_box_pack_start(GTK_BOX(dialog_action), p2->table, TRUE, TRUE, 0);
 	
-	label = gtk_label_new(_("The <i>before</i> text will be inserted before the cursor position or the current selection, the <i>after</i> text will be inserted after the cursor position or the current selection. You may use %0, %1, ...%9 placeholders to ask for values when you activate this item. Give these placeholders an appropriate name on the right."));
+	label = gtk_label_new(_("The <i>before</i> text will be inserted before the cursor position or the current selection, the <i>after</i> text will be inserted after the cursor position or the current selection. You may use %0, %1, ...%9 placeholders to ask for values when you activate this item. Give these placeholders an appropriate name on the right. (Please use %% if you need literal % in your string!)"));
 	gtk_label_set_use_markup(GTK_LABEL(label),TRUE);
 	gtk_label_set_line_wrap(GTK_LABEL(label),TRUE);
 	gtk_table_attach(GTK_TABLE(p2->table),label, 0,3,0,1
@@ -377,7 +369,7 @@ static gpointer snippets_build_pageInsert(Tsnipwiz *snwiz, GtkWidget *dialog_act
 		p2->entries[i] = gtk_entry_new();
 		gtk_table_attach(GTK_TABLE(p2->table),p2->entries[i], 2,3,i+2,i+3
 					,GTK_FILL,GTK_FILL,0,0);
-		p2->is_file[i] = gtk_toggle_button_new();
+		p2->is_file[i] = gtk_check_button_new();
 		gtk_table_attach(GTK_TABLE(p2->table),p2->is_file[i], 3,4,i+2,i+3
 					,GTK_SHRINK,GTK_SHRINK,0,0);
 	}
@@ -451,7 +443,7 @@ static gint snippets_test_pageInsert(Tsnipwiz *snwiz, gpointer data) {
 	before = textbuffer_get_all_chars(p2->before);	
 	after = textbuffer_get_all_chars(p2->after);		
 	xmlSetProp(childn, (const xmlChar *)"title", (const xmlChar *)snwiz->name);
-	if (strlen(snwiz->description)) {
+	if (snwiz->description) {
 		xmlSetProp(childn, (const xmlChar *)"tooltip", (const xmlChar *)snwiz->description);
 	}
 	
@@ -753,7 +745,7 @@ void snippets_new_item_dialog(Tsnippetswin *snw, xmlNodePtr node) {
 	snwiz = g_new0(Tsnipwiz,1);
 	snwiz->snw = snw;
 	snwiz->node = node;
-	snwiz->dialog = gtk_dialog_new_with_buttons(_("New snippet"),GTK_WINDOW(snw->bfwin->main_window),
+	snwiz->dialog = gtk_dialog_new_with_buttons(node? _("Edit snippet") :_("New snippet"),GTK_WINDOW(snw->bfwin->main_window),
 					GTK_DIALOG_DESTROY_WITH_PARENT,
 					GTK_STOCK_CANCEL,GTK_RESPONSE_REJECT,
 					GTK_STOCK_GO_FORWARD,1,
